PATCH = 'PATCH'  # RFC 5789

CONNECT = 'CONNECT'  # RFC 2616
DELETE = 'DELETE'  # RFC 2616
GET = 'GET'  # RFC 2616
OPTIONS = 'OPTIONS'  # RFC 2616
HEAD = 'HEAD'  # RFC 2616
POST = 'POST'  # RFC 2616
PUT = 'PUT'  # RFC 2616
TRACE = 'TRACE'  # RFC 2616


COPY = 'COPY'  # RFC 2518 (WebDAV)
LOCK = 'LOCK'  # RFC 2518 (WebDAV)
MKCOL = 'MKCOL'  # RFC 2518 (WebDAV)
MOVE = 'MOVE'  # RFC 2518 (WebDAV)
PROPFIND = 'PROPFIND'  # RFC 2518 (WebDAV)
PROPPATCH = 'PROPPATCH'  # RFC 2518 (WebDAV)
UNLOCK = 'UNLOCK'  # RFC 2518 (WebDAV)

VERSION_CONTROL = 'VERSION-CONTROL'  # RFC 3253 (WebDAV Extension)
REPORT = 'REPORT'  # RFC 3253 (WebDAV Extension)
CHECKOUT = 'CHECKOUT'  # RFC 3253 (WebDAV Extension)
CHECKIN = 'CHECKIN'  # RFC 3253 (WebDAV Extension)
UNCHECKOUT = 'UNCHECKOUT'  # RFC 3253 (WebDAV Extension)
MKWORKSPACE = 'MKWORKSPACE'  # RFC 3253 (WebDAV Extension)
UPDATE = 'UPDATE'  # RFC 3253 (WebDAV Extension)
LABEL = 'LABEL'  # RFC 3253 (WebDAV Extension)
MERGE = 'MERGE'  # RFC 3253 (WebDAV Extension)
BASELINE_CONTROL = 'BASELINE-CONTROL'  # RFC 3253 (WebDAV Extension)
MKACTIVITY = 'MKACTIVITY'  # RFC 3253 (WebDAV Extension)

ORDERPATCH = 'ORDERPATCH'  # RFC 3648 (WebDAV Order Collection Spaecification)

ACL = 'ACL'  # RFC 3744 (WebDAV Access Control)
